/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from '@angular/core';
import {NgRedux} from 'ng2-redux';
import {ConnectionHandler} from "../ConnectionHandler";
import {RepetierState} from '../state/RepetierState';
import {
    PrinterState,
    GroupState,
    GCodeFileState,
    TemperatureDataState,
    INITIAL_GCODE_FILE_STATE
} from "../state/PrinterState";
import {RSUtils, Deferred, SerializationHelper} from "../RSUtils"
import {RSBrowseFolder} from "../BrowseFolder";
import {ServerFolderState} from "../state/ServerState";
import {VisualAids} from "../VisualAids";

export interface ScriptName {
    name:string;
    title:string;
}

@Injectable()
export class PrinterAction {

    static SET_CONFIG: string = 'PRINTER_SET_CONFIG';
    static SET_GROUPS: string = 'PRINTER_SET_GROUPS';
    static UPDATE_STATE: string = 'PRINTER_UPDATE_STATE';
    static UPDATE_STATUS: string = 'PRINTER_UPDATE_STATUS';
    static UPDATE_GCODE: string = 'PRINTER_UPDATE_GCODE';
    static SET_IMG_URL: string = 'PRINTER_SET_IMG_URL';
    static UPDATE_IMG_URLS: string = 'PRINTER_UPDATE_IMG_URLS';
    static SET_PRINTQUEUE: string = 'PRINTER_SET_PRINTQUEUE';
    static SET_MODELS: string = 'PRINTER_SET_MODELS';
    static UPDATE_GCODE_FILTER: string = 'PRINTER_UPDATE_GCODE_FILTER';
    static ADD_BED_TEMPERATURE: string = 'PRINTER_ADD_BED_TEMPERATURE';
    static ADD_CHAMBER_TEMPERATURE: string = 'PRINTER_ADD_CHAMBER_TEMPERATURE';
    static ADD_EXTRUDER_TEMPERATURE: string = 'PRINTER_ADD_EXTRUDER_TEMPERATURE';
    static UPDATE_SCRIPT: string = 'UPDATE_SCRIPT';
    static UPDATE_LOG_STATUS: string = 'PRINTER_UPDATE_LOG_STATUS';
    static ADD_LOG_LINE: string = 'PRINTER_ADD_LOG_LINE';
    static RESET_IMAGE_REFERENCE: string = 'PRINTER_RESET_IMAGE_REFERENCE';

    static scriptNames:ScriptName[] = [{name: "start", title: "<?php _('Run before job') ?>"},
        {name: "end", title: "<?php _('Run after job') ?>"},
        {name: "kill", title: "<?php _('Run if job is aborted') ?>"},
        {name: "pause", title: "<?php _('Run on pause') ?>"},
        {name: "continue", title: "<?php _('Run on continue') ?>"},
        {name: "connect", title: "<?php _('Run on connect') ?>"},
        {name: "remove_filament", title: "<?php _('Remove filament') ?>"},
        {name: "insert_filament", title: "<?php _('Insert filament') ?>"}
    ];

    constructor(private redux: NgRedux<RepetierState>,
                private connectionHandler: ConnectionHandler,
                private visualAid: VisualAids) {
    }

    getPrinterState(slug: string): PrinterState {
        return this.redux.getState().printers.printers[slug];
    }

    getPrinterConfig(slug: string): any {
        let p: PrinterState = this.redux.getState().printers.printers[slug];
        return p ? p.config : undefined;
    }

    static makeType(slug: string, tp: string): string {
        return "printer/" + slug + "/" + tp;
    }

    dispatch(slug: string, tp: string, data: any): any {
        this.redux.dispatch({
            type: PrinterAction.makeType(slug, tp),
            payload: data
        });
    }

    updateScript(slug:string, data:any):void {
        this.dispatch(slug, PrinterAction.UPDATE_SCRIPT,data);
    }

    resetImageReference(slug:string):void {
        this.dispatch(slug, PrinterAction.RESET_IMAGE_REFERENCE,{});
    }

    setConfigData(slug: string, data: any): void {
        this.dispatch(slug, PrinterAction.SET_CONFIG, {config: data});
    }

    setGroups(slug: string, groups: {[key: string]: GroupState}, activeGroup: string) {
        this.dispatch(slug, PrinterAction.SET_GROUPS, {groups: groups, activeGroup: activeGroup});
    }

    setActiveGroupId(slug: string, id: string): void {
        this.dispatch(slug, PrinterAction.UPDATE_STATE, {state: {activeGroupId: id}});
    }

    updateState(slug: string, data: any): void {
        this.dispatch(slug, PrinterAction.UPDATE_STATE, {state: data});
    }

    updateStatus(slug: string, data: any): void {
        this.dispatch(slug, PrinterAction.UPDATE_STATUS, {slug: slug, status: data});
    }

    updateGCode(data: GCodeFileState): void {
        this.dispatch(data.slug, PrinterAction.UPDATE_GCODE, data);
    }

    updateImageUrls(slug: string, data: any): void {
        this.dispatch(slug, PrinterAction.UPDATE_IMG_URLS, data);
    }

    addBedTemperature(slug: string, temp: TemperatureDataState): void {
        this.dispatch(slug, PrinterAction.ADD_BED_TEMPERATURE, temp);
    }

    addChamberTemperature(slug: string, temp: TemperatureDataState): void {
        this.dispatch(slug, PrinterAction.ADD_CHAMBER_TEMPERATURE, temp);
    }

    addExtruderTemperature(slug: string, extrId: number, temp: TemperatureDataState): void {
        this.dispatch(slug, PrinterAction.ADD_EXTRUDER_TEMPERATURE, {extruderId: extrId, temp: temp});
    }

    // ----- following commands run server queries -----

    updateData(slug: string): void {
        this.updateGroups(slug); // models use groups, so create them first
        this.gcodeFetchModels(slug);
        this.gcodeFetchPrintqueue(slug);
        this.updateConfig(slug);
        this.fetchScripts(slug);
    }

    updateConfig(slug: string): void {
        this.connectionHandler.send("getPrinterConfig", {}, slug).then((data) => {
            this.setConfigData(slug, data);
        });
    }

    storeNewConfig(slug:string, config:any):Promise<any> {
        return this.connectionHandler.send("setPrinterConfig",config,slug);
    }

    updateGroups(slug: string): void {
        this.connectionHandler.send("listModelGroups", {}, slug).then((data: any) => {
            let hasGroup = false;
            let active = this.redux.getState().printers.printers[slug].activeGroupId;
            let groups: {[key: string]: GroupState} = {'*': {id: '*', name: "<?php _('Group: All') ?>"}};
            for (let idx in data.groupNames) {
                if (!data.groupNames.hasOwnProperty(idx)) continue;
                let v = data.groupNames[idx];
                hasGroup = hasGroup || v == active;
                if (v == "#")
                    groups["#"] = ({id: "#", name: "<?php _('Group: Default') ?>"});
                else
                    groups[v] = ({id: v, name: "<?php _('Group:') ?> " + v});
            }
            if (!hasGroup) {
                active = "#";
            }
            this.setGroups(slug, groups, active);
        });
    }

    // ----- gcode related functions -----

    gcodeInitPricing(gcodeOrig: GCodeFileState): GCodeFileState {
        let gcode: GCodeFileState = Object.assign({}, gcodeOrig);
        let p: any = this.redux.getState().server.settings["pricing"];
        let map = p.map;
        gcode.perExtruder = [];

        function filamentIndex(uid) {
            if (!uid) return 0;
            let idx = 0;
            for (let i in p.filaments) {
                let f = p.filaments[i];
                if (f.u == uid)
                    idx = parseInt(i);
            }
            return idx;
        }

        gcode.currency = p.currency;
        for (let idx in gcode.volumeUsage) {
            let vol = gcode.volumeUsage[idx];
            if (vol <= 0) continue;
            gcode.perExtruder.push({
                extruder: parseInt(idx) + 1,
                mm: gcode.extruderUsage[idx],
                volume: vol * 0.001, price: 0,
                filament: p.filaments[filamentIndex(map[idx + "_" + gcode.slug])],
                weight: 0
            });
        }
        return this.gcodeUpdatePricing(gcode);
    }

    gcodeUpdatePricing(gcodeOrig: GCodeFileState): GCodeFileState {
        let gcode: GCodeFileState = Object.assign({}, gcodeOrig);
        let p: any = this.redux.getState().server.settings["pricing"];
        gcode.weightTotal = 0;
        gcode.total = p.perPrint + p.perHour * gcode.printTime / 3600;
        for (let idx in gcode.perExtruder) {
            let f = gcode.perExtruder[idx];
            f.weight = f.volume * f.filament.w;
            gcode.weightTotal += f.weight;
            f.price = f.weight * f.filament.p / 1000.0;
            gcode.total += f.price;
        }
        return gcode;
    }

    gcodeDelete(gcode: GCodeFileState): void {
        if (gcode.list == "models") {
            this.connectionHandler.send("removeModel", {id: gcode.id});
        } else {
            this.connectionHandler.send("removeJob", {id: gcode.id});
        }
    }

    gcodePrint(gcode: GCodeFileState) {
        if (gcode.type == 1) {
            this.connectionHandler.send("startJob", {id: gcode.id}, gcode.slug);
        } else {
            this.connectionHandler.send("copyModel", {id: gcode.id}, gcode.slug);
        }
    }

    gcodeImageUrl(slug: string, queue, id: number, sz: string) {
        if (typeof(id) == "undefined") {
            return "/mod/server/img/rendering_" + sz + ".png";
        }
        let key = slug + "_" + queue + id + sz;
        if (!this.redux.getState().printers.printers[slug].imageUrls[key]) {
            let url: string = "/dyn/render_image?q=" + queue + "&id=" + id + "&slug=" + slug + "&t=" + sz; // + "&tm=" + new Date().getTime();
            if(queue == "jobs") {
                url += "&tm=" + new Date().getTime();
            }
            this.redux.dispatch({
                type: 'printer/' + slug + '/' + PrinterAction.SET_IMG_URL,
                payload: {key: key, url: url}
            });
        }
        if (this.redux.getState().license.supportPreviews) {
            return this.redux.getState().printers.printers[slug].imageUrls[key];
        }
        return "/img/pro_" + sz + ".png";
    }

    gcodeGetLatestVersion(g: GCodeFileState): GCodeFileState {
        if (!g)
            return undefined;
        if (g.list == "jobs") {
            for (let x of this.redux.getState().printers.printers[g.slug].queue) {
                if (x.id == g.id)
                    return x;
            }
        } else {
            for (let x of this.redux.getState().printers.printers[g.slug].allModels) {
                if (x.id == g.id)
                    return x;
            }
        }
        return undefined;
    }

    gcodeFetchPrintqueue(slug: string): void {
        this.connectionHandler.send("listJobs", {}, slug).then((r) => {
            let queue: GCodeFileState[] = [];
            for (let val2 of r.data) {
                let val: GCodeFileState = SerializationHelper.convert(RSUtils.copyObject(INITIAL_GCODE_FILE_STATE), val2);

                val.error = val.analysed == 2;
                val.processing = val.analysed == 0;
                val.ok = val.analysed == 1;
                val.list = "jobs";
                val.slug = slug;
                val.imgL = this.gcodeImageUrl(slug, "jobs", val.id, "l");
                val.imgM = this.gcodeImageUrl(slug, "jobs", val.id, "m");
                val.imgT = this.gcodeImageUrl(slug, "jobs", val.id, "s");
                val.type = 1;
                queue.push(this.gcodeInitPricing(val));
            }
            this.dispatch(slug, PrinterAction.SET_PRINTQUEUE, queue);
        });
    }


    gcodeFetchModels(slug: string): void {
        this.connectionHandler.send("listModels", {}, slug).then((r: any) => {
            let allModels = [];
            for (let val2 of r.data) {
                let val: GCodeFileState = SerializationHelper.convert(RSUtils.copyObject(INITIAL_GCODE_FILE_STATE), val2);

                val.error = val.analysed == 2;
                val.processing = val.analysed == 0;
                val.ok = val.analysed == 1;
                val.list = "models";
                val.slug = slug;
                val.imgL = this.gcodeImageUrl(slug, "models", val.id, "l");
                val.imgM = this.gcodeImageUrl(slug, "models", val.id, "m");
                val.imgT = this.gcodeImageUrl(slug, "models", val.id, "s");
                val.type = 2;
                allModels.push(this.gcodeInitPricing(val));
            }
            this.dispatch(slug, PrinterAction.SET_MODELS, allModels);
        });
    }

    extruderName(slug: string, idx: number): string {
        let config = this.redux.getState().printers.printers[slug].config;
        if (config.extruders) {
            let alias = config.extruders[idx].alias;
            if (alias) return alias;
        }
        return "<?php _('Extruder') ?> " + (idx + 1);
    }

    importFiles(slug: string, folder: RSBrowseFolder, files: ServerFolderState[], asJob: boolean) {
        let arr = [];
        for (let i of files) {
            arr.push({name: i.name, url: "folder://" + folder.selectedFolder + i.path, job: asJob});
        }
        if (arr.length == 0) return;
        let printer: PrinterState = this.redux.getState().printers.printers[slug];
        this.connectionHandler.send("importURL", {files: arr, folder: printer.activeGroupId}).then((data) => {
            if (data.ok) {
                this.visualAid.bindProgressMsgToId(data.msgId, "Uploading Files");
                this.visualAid.updateProgress({msgId: data.msgId, finished: false, progress: 0, ok: true});
            }
        }, () => {
        });
    }

    motorsOff(slug: string): void {
        this.connectionHandler.send("motorsOff", {}, slug);
    }

    updateGCodeQueues(slug: string) {
        this.gcodeFetchModels(slug);
        this.gcodeFetchPrintqueue(slug);
    }


    hasFirmwareSettings: boolean = false;
    firmwareSettings: any = {};

    setConfigFromFirmware(slug: string): void {
        let p: PrinterState = this.getPrinterState(slug);
        let c: any = RSUtils.copyObject(p.config); //RSPrinterConfig.config;
        let f: any = p.firmwareSettings;
        if (f.hasOwnProperty("KeepAliveInterval") && f.KeepAliveInterval > 1000)
            c.connection.serial.communicationTimeout = Math.round(f.KeepAliveInterval / 1000);
        if (f.hasOwnProperty("baudrate"))
            c.connection.serial.baudrate = f.baudrate;
        if (f.hasOwnProperty("InputBuffer"))
            c.connection.serial.inputBufferSize = f.InputBuffer;
        if (f.hasOwnProperty("PrintlineCache"))
            c.movement.movebuffer = f.PrintlineCache;
        if (f.hasOwnProperty("maxXYSpeed"))
            c.movement.maxXYSpeed = f.maxXYSpeed;
        if (f.hasOwnProperty("xyJerk"))
            c.movement.xyJerk = f.xyJerk;
        if (f.hasOwnProperty("maxZSpeed"))
            c.movement.maxZSpeed = f.maxZSpeed;
        if (f.hasOwnProperty("zJerk")) {
            if (f.zJerk > 0)
                c.movement.zJerk = f.zJerk;
            else if (f.xyJerk > 0)
                c.movement.zJerk = f.xyJerk;
        }
        if (f.hasOwnProperty("xyPrintAcceleration"))
            c.movement.xyPrintAcceleration = f.xyPrintAcceleration;
        if (f.hasOwnProperty("xyTravelAcceleration"))
            c.movement.xyTravelAcceleration = f.xyTravelAcceleration;
        if (f.hasOwnProperty("zPrintAcceleration"))
            c.movement.zPrintAcceleration = f.zPrintAcceleration;
        if (f.hasOwnProperty("zTravelAcceleration"))
            c.movement.zTravelAcceleration = f.zTravelAcceleration;
        if (f.hasOwnProperty("ZLength"))
            c.movement.zMax = f.ZLength;
        if (f.hasOwnProperty("RetractionSpeed"))
            c.movement.G10Speed = f.RetractionSpeed;
        if (f.hasOwnProperty("RetractionLength"))
            c.movement.G10Distance = f.RetractionLength;
        if (f.hasOwnProperty("RetractionLongLength"))
            c.movement.G10LongDistance = f.RetractionLongLength;
        if (f.hasOwnProperty("RetractionUndoSpeed"))
            c.movement.G11Speed = f.RetractionUndoSpeed;
        if (f.hasOwnProperty("RetractionUndoExtraLength"))
            c.movement.G11ExtraDistance = f.RetractionUndoExtraLength;
        if (f.hasOwnProperty("RetractionUndoExtraLongLength"))
            c.movement.G11ExtraLongDistance = f.RetractionUndoExtraLongLength;
        if (f.hasOwnProperty("RetractionZLift"))
            c.movement.G10ZLift = f.RetractionZLift;
        if (f.hasOwnProperty("printerType")) {
            c.general.printerVariant = "cartesian";
            if (f.printerType == 2) {
                c.general.printerVariant = "delta";
                c.shape.basicShape.shape = "circle";
                if (f.hasOwnProperty("printableRadius")) {
                    c.shape.basicShape.radius = f.printableRadius;
                    c.shape.basicShape.xMin = c.movement.xMin = -f.printableRadius;
                    c.shape.basicShape.xMax = c.movement.xMax = f.printableRadius;
                    c.shape.basicShape.yMin = c.movement.yMin = -f.printableRadius;
                    c.shape.basicShape.yMax = c.movement.yMax = f.printableRadius;
                }
            }
            if (f.printerType == 1) {
                if (f.hasOwnProperty("XMin")) {
                    c.shape.basicShape.xMin = c.movement.xMin = f.XMin;
                    if (f.hasOwnProperty("XLength"))
                        c.shape.basicShape.xMax = c.movement.xMax = f.XMin + f.XLength;
                }
                if (f.hasOwnProperty("YMin")) {
                    c.shape.basicShape.yMin = c.movement.yMin = f.YMin;
                    if (f.hasOwnProperty("YLength"))
                        c.shape.basicShape.yMax = c.movement.yMax = f.YMin + f.YLength;
                }
                if (f.hasOwnProperty("ZMin")) {
                    c.shape.basicShape.zMin = c.movement.zMin = f.ZMin;
                    if (f.hasOwnProperty("ZLength"))
                        c.shape.basicShape.zMax = c.movement.zMax = f.ZMin + f.ZLength;
                }
            }
        }
        if (f.hasOwnProperty("hasHeatedBed"))
            c.heatedBed.installed = Boolean(f.hasHeatedBed);
        if (f.hasOwnProperty("maxBedTemp"))
            c.heatedBed.maxTemp = f.maxBedTemp;
        if (f.hasOwnProperty("hasHeatedChamber"))
            c.heatedChamber.installed = Boolean(f.hasHeatedChamber);
        if (f.hasOwnProperty("maxChamberTemp"))
            c.heatedChamber.maxTemp = f.maxChamberTemp;
        if (f.hasOwnProperty("sdInstalled"))
            c.general.sdcard = Boolean(f.sdInstalled);
        if (f.hasOwnProperty("fanInstalled"))
            c.general.fan = Boolean(f.fanInstalled);
        if (f.hasOwnProperty("fan2Installed"))
            c.general.fan2 = Boolean(f.fan2Installed);
        if (f.hasOwnProperty("softwarePowerSwitch"))
            c.general.softwarePower = Boolean(f.softwarePowerSwitch);
        for (let idx: number = 0; idx < c.extrudes.length; idx++) {
            let ex = c.extruders[idx];
            let app = "[" + idx + "]";
            if (f.hasOwnProperty("extrJerk" + app))
                ex.eJerk = f["extrJerk" + app];
            if (f.hasOwnProperty("extrMaxSpeed" + app))
                ex.maxSpeed = f["extrMaxSpeed" + app];
            if (f.hasOwnProperty("extrMaxTemp" + app))
                ex.maxTemp = f["extrMaxTemp" + app];
            if (f.hasOwnProperty("extrAcceleration" + app))
                ex.acceleration = f["extrAcceleration" + app];

        }
    }

    private readingFirmware: boolean = false;
    private firmwareSettingsPromise: Deferred = null;

    catchFirmwareSettings = (slug: string) => {
        this.connectionHandler.send("getFirmwareSettings", {}, slug).then((data) => {
            this.firmwareSettings = data.settings;
            this.hasFirmwareSettings = true;
            //log("New firmware settings", Service.firmwareSettings);
            this.setConfigFromFirmware(slug);
            this.readingFirmware = false;
            this.connectionHandler.fireLocalEvent("firmwareConfigRead", this.firmwareSettings);
            if (this.firmwareSettingsPromise != null) {
                this.firmwareSettingsPromise.resolve(this.firmwareSettings);
                this.firmwareSettingsPromise = null;
            }
        });
    };

    updateNewPrinter(slug: string) {
        this.updateState(slug, {
            hasFirmwareSettings: false,
            firmwareSettings: []
        });
        this.connectionHandler.send("getFirmwareSettings", {}, slug).then((data: any) => {
            this.updateState(slug, {
                hasFirmwareSettings: true,
                firmwareSettings: data.settings
            });
            this.connectionHandler.fireLocalEvent("firmwareConfigRead", data.settings, this.getPrinterState(slug));
        });
    }

    updateFirmwareAutoset(slug: string) {
        this.connectionHandler.fireLocalEvent("firmwareConfigRead", this.firmwareSettings, this.getPrinterState(slug));
    }

    updateFirmwareSettings(slug: string) {
        this.readingFirmware = true;
        this.firmwareSettingsPromise = new Deferred();
        this.connectionHandler.send("startUpdateFirmwareSettings", {}, slug).then(() => {
            setTimeout(() => this.catchFirmwareSettings(slug), 3000);
        });
        return this.firmwareSettingsPromise.promise;
    }

    setExtruderTemperature(slug: string, extr: number, temp: number) {
        this.connectionHandler.send("setExtruderTemperature", {temperature: temp, extruder: extr}, slug);
    }

    setBedTemperature(slug: string, temp) {
        this.connectionHandler.send("setBedTemperature", {temperature: temp}, slug);
    }

    setChamberTemperature(slug: string, temp) {
        this.connectionHandler.send("setChamberTemperature", {temperature: temp}, slug);
    }


    power(slug: string) {
        if (this.getPrinterState(slug).powerOn) {
            this.connectionHandler.send("send", {cmd: "M81"});
        } else {
            this.connectionHandler.send("send", {cmd: "M80"}, slug);
        }
    };

    sendGCode(slug:string, cmd:string):void {
        this.connectionHandler.send("send", {cmd: cmd}, slug);
    }

    emergencyStop(slug: string) {
        this.connectionHandler.send("emergencyStop", {}, slug);
    }

    setSpeedMultiply(slug: string, diff: number) {
        this.updateState(slug, {speedMultiply: diff});
        this.connectionHandler.send("setSpeedMultiply", {speed: diff}, slug);
    };

    setFlowMultiply(slug: string, diff: number) {
        this.updateState(slug, {flowMultiply: diff});
        this.connectionHandler.send("setFlowMultiply", {speed: diff}, slug);
    };

    setFanPercent(slug: string, newSpeed: number) {
        this.updateState(slug, {fanPercent: newSpeed});
        this.connectionHandler.send("setFanSpeed", {speed: Math.round(newSpeed * 2.55)}, slug);
    };

    setFanOn(slug: string, on: boolean): void {
        let p: PrinterState = this.getPrinterState(slug);
        let percent: number = p.fanPercent;
        if (percent == 0)
            percent = 100;
        this.updateState(slug, {fanOn: on, fanPercent: percent});
        this.connectionHandler.send("setFanSpeed", {speed: Math.round(percent * 2.55), "on": on}, slug);
    };

    setFan2Percent(slug: string, newSpeed: number) {
        this.updateState(slug, {fan2Percent: newSpeed});
        this.connectionHandler.send("setFan2Speed", {speed: Math.round(newSpeed * 2.55)}, slug);
    };

    setFan2On(slug: string, on: boolean): void {
        let p: PrinterState = this.getPrinterState(slug);
        let percent: number = p.fan2Percent;
        if (percent == 0)
            percent = 100;
        this.updateState(slug, {fan2On: on, fa2nPercent: percent});
        this.connectionHandler.send("setFan2Speed", {speed: Math.round(percent * 2.55), "on": on}, slug);
    };

    toggleCaseLights(slug: string): void {
        if (this.getPrinterState(slug).lightsOn)
            this.connectionHandler.send("setCaseLights", {on: false}, slug);
        else
            this.connectionHandler.send("setCaseLights", {on: true}, slug);
    };

    xMoveRel(slug: string, x: number): void {
        if (this.getPrinterConfig(slug).movement.invertX)
            x = -x;
        //this.posX += x;
        this.connectionHandler.send("move", {x: x, relative: true}, slug);
        setTimeout(()=> {
            this.connectionHandler.fireLocalEvent("updateStates");
        }, 20);
    }

    yMoveRel(slug: string, y: number): void {
        let p: PrinterState = this.getPrinterState(slug);
        if (p.config.movement.invertY)
            y = -y;
        this.connectionHandler.send("move", {y: y, relative: true}, slug);
        setTimeout(()=> {
            this.connectionHandler.fireLocalEvent("updateStates");
        }, 20);
    }

    zMoveRel(slug: string, z: number): void {
        let p: PrinterState = this.getPrinterState(slug);
        if (p.config.movement.invertZ)
            z = -z;
        this.connectionHandler.send("move", {z: z, relative: true}, slug);
        setTimeout(()=> {
            this.connectionHandler.fireLocalEvent("updateStates");
        }, 20);
    };

    xyMoveRel(slug: string, x: number, y: number): void {
        let p: PrinterState = this.getPrinterState(slug);
        if (p.config.movement.invertX)
            x = -x;
        if (p.config.movement.invertY)
            y = -y;
        this.connectionHandler.send("move", {x: x, y: y, relative: true}, slug);
        setTimeout(()=> {
            this.connectionHandler.fireLocalEvent("updateStates");
        }, 20);
    }

    eMoveRel(slug: string, x: number, speed: number): void {
        this.connectionHandler.send("move", {e: x, relative: true, speed: speed}, slug);
        setTimeout(()=> {
            this.connectionHandler.fireLocalEvent("updateStates");
        }, 20);
    };

    selectExtruder(slug: string, extr: number): void {
        this.connectionHandler.send("send", {cmd: "T" + extr + "\nG92 E0"}, slug);
        setTimeout(()=> {
            this.connectionHandler.fireLocalEvent("updateStates");
        }, 20);
    }

    activate(slug: string):void {
        this.connectionHandler.send("activate", {printer: slug}, slug);
    }

    deactivate(slug: string):void {
        this.connectionHandler.send("deactivate", {printer: slug}, slug);
    }

    stopPrint(slug: string):void {
        let p: PrinterState = this.getPrinterState(slug);
        if (p.jobid)
            this.connectionHandler.send("stopJob", {id: p.jobid});
    }

    pausePrint(slug: string): void {
        this.updateState(slug, {paused: true});
        this.connectionHandler.send("send", {cmd: "@pause <?php _('User requested pause.') ?>"}, slug);
    }

    unpausePrint(slug: string): void {
        // this.updateState(slug, {paused: false});
        this.connectionHandler.send("continueJob", {}, slug);
    }

    executeNamedFunction(slug: string, f: string, value: number): void {
        let p: PrinterState = this.getPrinterState(slug);
        switch (f) {
            case "xLeft":
                this.xMoveRel(slug, -p.xyzStepSize);
                break;
            case "xRight":
                this.xMoveRel(slug, p.xyzStepSize);
                break;
            case "yUp":
                this.yMoveRel(slug, p.xyzStepSize);
                break;
            case "yUpXLeft":
                this.xyMoveRel(slug, -p.xyzStepSize, p.xyzStepSize);
                break;
            case "yUpXRight":
                this.xyMoveRel(slug, p.xyzStepSize, p.xyzStepSize);
                break;
            case "yDown":
                this.yMoveRel(slug, -p.xyzStepSize);
                break;
            case "yDownXLeft":
                this.xyMoveRel(slug, -p.xyzStepSize, -p.xyzStepSize);
                break;
            case "yDownXRight":
                this.xyMoveRel(slug, p.xyzStepSize, -p.xyzStepSize);
                break;
            case "zUp":
                this.zMoveRel(slug, p.xyzStepSize);
                break;
            case "zDown":
                this.zMoveRel(slug, -p.xyzStepSize);
                break;
            case "eExtrude":
                this.eMoveRel(slug, p.eStepSize, p.extrudeSpeed);
                break;
            case "eRetract":
                this.eMoveRel(slug, -p.eStepSize, p.extrudeSpeed);
                break;
            case "eExtrudeFast":
                this.eMoveRel(slug, p.eStepSize, p.retractSpeed);
                break;
            case "eRetractFast":
                this.eMoveRel(slug, -p.eStepSize, p.retractSpeed);
                break;
            case "home":
                this.connectionHandler.send("send", {cmd: "G28"}, slug);
                break;
            case "homeX":
                this.connectionHandler.send("send", {cmd: "G28 X0"}, slug);
                break;
            case "homeY":
                this.connectionHandler.send("send", {cmd: "G28 Y0"}, slug);
                break;
            case "homeZ":
                this.connectionHandler.send("send", {cmd: "G28 Z0"}, slug);
                break;
            case "motorsOff":
                this.motorsOff(slug);
                break;
        }
    }

    printingImage(slug: string, imgSize: string):string {
        let p: PrinterState = this.getPrinterState(slug);
        if (!p || !p.printing) return "";
        return this.gcodeImageUrl(slug, 'jobs', p.jobid, imgSize || 'l');
    };

    fetchScripts(slug: string):void {
        for(let i=0;i < PrinterAction.scriptNames.length;i++) {
            this.connectionHandler.send("getScript",{name:PrinterAction.scriptNames[i].name},slug).then((s:any)=>{
                this.updateScript(slug,s);
            },()=>{})
        }
    }

    runScript(slug:string,scriptName:string):void {
        this.connectionHandler.send("runScript",{script:scriptName},slug);
    }

    getLoglevel(slug:string):number {
        let p: PrinterState = this.getPrinterState(slug);
        return 12 + (p.log.logCommands ? 1 : 0) + (p.log.logACK ? 2 : 0);
    }
    sendLogLevel(slug:string):void {
        this.connectionHandler.send("setLoglevel",{logPrinter:slug,level:this.getLoglevel(slug)});
    }
    setLogCommands(slug:string, on:boolean):void {
        this.dispatch(slug,PrinterAction.UPDATE_LOG_STATUS,{logCommands:on});
        this.sendLogLevel(slug);
    }
    setLogACK(slug:string, on:boolean):void {
        this.dispatch(slug,PrinterAction.UPDATE_LOG_STATUS,{logACK:on});
        this.sendLogLevel(slug);
    }
    setLogPause(slug:string, on:boolean):void {
        this.dispatch(slug,PrinterAction.UPDATE_LOG_STATUS,{paused:on});
    }

}