/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from '@angular/core';
import {NgRedux} from 'ng2-redux';
import {ConnectionHandler} from "../ConnectionHandler";
import {RepetierState} from '../state/RepetierState';
import {PrinterErrorState} from '../state/PrinterErrorState';
import {VisualAids} from "../VisualAids"
@Injectable()
export class PrinterErrorAction {

    static ADD_ERROR: string = 'PERROR_ADD_ERROR';
    static UPDATE_ERROR: string = 'PERROR_UPDATE_ERROR';
    static MARK_SEEN: string = 'PERROR_MARK_SEEN';

    constructor(private redux: NgRedux<RepetierState>,
                private visualAids: VisualAids) {
    }

    markSeen(id: string) {
        this.redux.dispatch({type: PrinterErrorAction.MARK_SEEN, payload: id});
    }

    markAllSeen(): void {
        let closeMsg: boolean = false;
        let orig: {[key: string]: PrinterErrorState} = this.redux.getState().printerErrors;
        for (let idx in orig) {
            if (!orig.hasOwnProperty(idx)) continue;
            if (orig[idx].seen == false) {
                this.markSeen(idx);
                closeMsg = true;
            }
        }
        if (closeMsg) {
            this.visualAids.onPrinterErrorSolved();
        }
    }

    hasUnseenErrors(): boolean {
        let orig: {[key: string]: PrinterErrorState} = this.redux.getState().printerErrors;
        for (let idx in orig) {
            if (!orig.hasOwnProperty(idx)) continue;
            let e = orig[idx];
            if (!e.isError || e.seen) continue;
            return true;
        }
        return false;
    }

    static idBed(slug: string) {
        return "bed;" + slug;
    }

    static idChamber(slug: string) {
        return "chamber;" + slug;
    }

    static idExtruder(exid: number, slug: string) {
        return "" + exid + ";" + slug;
    }

    static create(_id: string, slug: string): PrinterErrorState {
        let parts = _id.split(";");
        return {
            id: _id,
            slug: slug,
            seen: true,
            errorId: 0,
            isError: false,
            stateString: '',
            device: parts[0] === "bed" ? "<?php _('Heated Bed') ?>" : "<?php _('Extruder') ?> " + parts[0]
        }
    }

    static errorToString(errorId: number): string {
        switch (errorId) {
            case 0:
                return "<?php _('No Error') ?>";
            case 1:
                return "<?php _('Sensor Defect') ?>";
            case 2:
                return "<?php _('Heater/Sensor Decoupled') ?>";
            case 3:
                return "<?php _('Filament Slipping') ?>";
            case 4:
                return "<?php _('Extruder Jammed') ?>";
            case 5:
                return "<?php _('Extruder Jammed/Out of Filament') ?>";
            case 6:
                return "<?php _('Out of Filament') ?>";
            default:
                return "<?php _('Unknown Error') ?>";
        }
    }

    createIfNeeded(id: string, slug: string): PrinterErrorState {
        let err: PrinterErrorState = this.redux.getState().printerErrors[id];
        if (!err) {
            err = PrinterErrorAction.create(id, slug);
            this.redux.dispatch({type: PrinterErrorAction.ADD_ERROR, payload: err});
        }
        return err;
    }

    updateHeaterError(id: string, e: number): boolean {
        let err: PrinterErrorState = this.redux.getState().printerErrors[id];
        if (!err) return false;
        if (e == err.errorId)
            return false;
        this.redux.dispatch({
            type: PrinterErrorAction.UPDATE_ERROR, payload: {
                id: id, error: {
                    isError: e !== 0,
                    errorId: e,
                    seen: e === 0,
                    stateString: PrinterErrorAction.errorToString(e)
                }
            }
        });
        this.visualAids.onPrinterError();
        return true;
    }

    static listErrors(orig: {[key: string]: PrinterErrorState}): PrinterErrorState[] {
        let a: PrinterErrorState[] = [];
        for (let idx in orig) {
            if (!orig.hasOwnProperty(idx)) continue;
            let e = orig[idx];
            if (!e.isError || e.seen) continue;
            a.push(e);
        }
        a.sort(function (a: PrinterErrorState, b: PrinterErrorState) {
            let o1 = a.device.toLowerCase();
            let o2 = b.device.toLowerCase();
            if (o1 < o2) return -1;
            if (o1 > o2) return 1;
            if (a.errorId < b.errorId) return 1;
            if (a.errorId > b.errorId) return -1;
            return 0;
        });
        return a;
    }
}
