/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {Injectable} from '@angular/core';
import {NgRedux} from 'ng2-redux';
import {ConnectionHandler,ServerEvent} from "../ConnectionHandler";
import {RepetierState} from '../state/RepetierState';
import {PrintersState} from "../state/PrintersState";
import {PrinterErrorAction} from "./PrinterErrorAction";
import {
    PrinterState,
    INITIAL_PRINTER_STATE,
    ToolState,
    INITAL_TOOL_STATE,
    TemperatureDataState
} from "../state/PrinterState";
import {PrinterAction} from "./PrinterAction";
import {VisualAids} from "../VisualAids";
import {RSUtils} from "../RSUtils";

@Injectable()
export class PrintersAction {

    static SET_ACTIVE: string = 'PRINTERS_SET_ACTIVE';
    static CREATE_PRINTER: string = 'PRINTERS_CREATE';
    static DESTROY_PRINTER: string = 'PRINTERS_DESTROY';
    static SET_AUTOSELECT: string = 'PRINTERS_SET_AUTOSELECT';
    static SET_STATE_TIME: string = 'PRINTER_SET_STATE_TIME';

    private updatingState = false;
    private updatingStatus = false;
    private sendMovesCounter:number = 0;

    constructor(private redux: NgRedux<RepetierState>,
                private connectionHandler: ConnectionHandler,
                private visualAid:VisualAids,
                private printerAction: PrinterAction,
                private printerErrorAction: PrinterErrorAction) {
        this.connectionHandler.connectionState.subscribe(connected => {
            if(connected) {
                this.updatingState = false;
                this.updatingStatus = false;
                if (this.sendMovesCounter > 0)
                    this.connectionHandler.send("sendMoves");
                let pList = this.redux.getState().printers.printers;
                for (let p in pList) {
                    if (pList.hasOwnProperty(p)) {
                        this.printerAction.sendLogLevel(p);
                        this.printerAction.resetImageReference(p);
                    }
                }
                this.updatePrinters();
            }
        });
        this.connectionHandler.localEvents.subscribe(this.localEventHandler);
    }

    private localEventHandler = (e:ServerEvent) => {
        switch(e.event) {
            case "updateStates":
                this.updateStates();
                break;
        }
    };

    setActiveSlug(slug: string): void {
        this.redux.dispatch({type: PrintersAction.SET_ACTIVE, payload: {slug: slug}});
    }

    setAutoselect(on:boolean): void {
        this.redux.dispatch({type: PrintersAction.SET_AUTOSELECT, payload: on});
    }

    addPrinter(printer: PrintersState): void {
        this.redux.dispatch({type: PrintersAction.CREATE_PRINTER, payload: printer});
    }

    delPrinter(slug: string): void {
        this.redux.dispatch({type: PrintersAction.DESTROY_PRINTER, payload: {slug: slug}});
    }

    // ----- following function query server -----

    private static preparePrinterListData(data): any {
        let out: any = {
            slug: data.slug,
            online: data.online,
            active: data.active,
            name: data.name
        };
        if (data.job === "none") {
            out.printJob = "";
            out.printing = false;
            out.paused = false;
            out.printDone = 0;
        } else {
            out.printJob = data.job;
            out.printing = true;
            out.printDone = data.done;
            out.jobid = data.jobid;
            out.paused = data.paused;
            out.pauseState = data.pauseState;
            out.ofLayer = data.ofLayer;
            out.printedTimeComp = data.printedTimeComp;
            out.printedTime = data.printTime;
            out.startTime = data.start;
        }
        out.idle = !out.printing || out.paused;
        return out;
    }

    public static prepareTemperature(temp: any): TemperatureDataState {
        return {
            output: temp.O,
            time: temp.t,
            tempSet: temp.S,
            temp: temp.T
        }
    }

    private static prepareToolFromState(tool: ToolState, data: any) {
        if (data.history) {
            for (let temp of data.history) {
                let newT:TemperatureDataState = PrintersAction.prepareTemperature(temp);
                while (tool.tempHistory.length > 0 && tool.tempHistory[0].time + 300000 < newT.time) {
                    tool.tempHistory.splice(0, 1);
                }
                tool.tempHistory.push(newT);
            }
        }
        tool.currentTemp = data.tempRead;
        tool.tempSet = data.tempSet;
        tool.output = data.output;
    }

    public prepareStateData(slug: string, data: any): any {
        let printer: PrinterState = this.redux.getState().printers.printers[slug];
        if (!printer) return {};
        let out: any = {};
        out.fanPercent = Math.floor(0.5 + data.fanVoltage / 2.55);
        out.fan2Percent = Math.floor(0.5 + data.fan2Voltage / 2.55);
        out.fan = data.fanVoltage;
        out.fanOn = data.fanOn;
        out.fan2 = data.fan2Voltage;
        out.fan2On = data.fan2On;
        out.activeExtruderId = data.activeExtruder;
        out.debugLevel = data.debugLevel;
        out.firmware = data.firmware;
        out.firmwareURL = data.firmwareURL;
        out.speedMultiply = data.speedMultiply;
        out.flowMultiply = data.flowMultiply;
        out.hasXHome = data.hasXHome;
        out.hasYHome = data.hasYHome;
        out.hasZHome = data.hasZHome;
        out.layer = data.layer;
        out.powerOn = data.powerOn;
        out.sdcardMounted = data.sdcardMounted;
        out.rec = data.rec;
        out.volumetric = data.volumetric;
        out.lightsOn = data.lights;
        out.posX = data.x;
        out.posY = data.y;
        out.posZ = data.z;
        out.extruders = RSUtils.copyArray(printer.extruders);
        out.heatedBed = RSUtils.copyObject(printer.heatedBed);
        out.heatedChamber = RSUtils.copyObject(printer.heatedChamber);
        let tempError: boolean = false;
        if (data.numExtruder < printer.numExtruder) {
            out.extruders.splice(data.numExtruder, out.extruders.length - data.numExtruder);
        }
        while (data.numExtruder > out.extruders.length) {
            let t: ToolState = RSUtils.copyObject(INITAL_TOOL_STATE);
            t.id = out.extruders.length;
            out.extruders.push(t);
        }
        out.numExtruder = data.numExtruder;
        for (let i = 0; i < out.numExtruder; i++) {
            PrintersAction.prepareToolFromState(out.extruders[i],data.extruder[i]);
            let bid = PrinterErrorAction.idExtruder(i, slug);
            this.printerErrorAction.createIfNeeded(bid,slug);
            tempError = tempError || this.printerErrorAction.updateHeaterError(bid,out.extruders[i].error);
        }
        if (data.heatedBed) {
            PrintersAction.prepareToolFromState(out.heatedBed,data.heatedBed);
            let bid = PrinterErrorAction.idBed(slug);
            this.printerErrorAction.createIfNeeded(bid,slug);
            tempError = tempError || this.printerErrorAction.updateHeaterError(bid,data.heatedBed.error);
        }
        if (data.heatedChamber) {
            PrintersAction.prepareToolFromState(out.heatedChamber,data.heatedChamber);
            let bid = PrinterErrorAction.idChamber(slug);
            this.printerErrorAction.createIfNeeded(bid,slug);
            tempError = tempError || this.printerErrorAction.updateHeaterError(bid,data.heatedChamber.error);
        }

        if (tempError) {
            if (this.printerErrorAction.hasUnseenErrors()) {
                this.visualAid.onPrinterError();
            } else {
                this.visualAid.onPrinterErrorSolved();
            }
        }
        return out;
    }

    updatePrinters = () => {
        if(this.updatingStatus)
            return; // no double query
        this.updatingStatus = true;
        this.connectionHandler.send("listPrinter").then((data) => {
            let exists: {[key: string]: boolean} = {};
            let printers: PrintersState = this.redux.getState().printers;
            for (let p in printers.printers) {
                if (printers.printers.hasOwnProperty(p))
                    exists[p] = false;
            }
            for (let p of data) {
                if (!printers.printers[p.slug]) {
                    exists[p.slug] = true;
                    this.addPrinter(Object.assign({}, INITIAL_PRINTER_STATE, PrintersAction.preparePrinterListData(p)));
                    this.printerAction.sendLogLevel(p.slug);
                    this.printerAction.updateData(p.slug);
                } else {
                    exists[p.slug] = true;
                    let newStatus = PrintersAction.preparePrinterListData(p);
                    if(RSUtils.newChanges(printers.printers[p.slug],newStatus)) {
                        this.printerAction.updateStatus(p.slug, newStatus);
                    }
                }
            }
            for (let key in exists) {
                if (!exists.hasOwnProperty(key)) continue;
                if (exists[key] === false) {
                    this.delPrinter(key);
                }
            }
            this.updatingStatus = false;
        });
    };

    updateStates = () => {
        if(this.updatingState)
            return;
        this.updatingState = true;
        this.connectionHandler.send("stateList", {includeHistory: this.redux.getState().printers.stateTime == 0}).then((data) => {
            let stateTime = new Date().getTime(); // Time of last state change
            let update:boolean = false;
            for (let key in data) {
                if (!data.hasOwnProperty(key)) continue;
                if(this.redux.getState().printers.printers[key]) {
                    let newState = this.prepareStateData(key,data[key]);
                    if(RSUtils.newChanges(this.redux.getState().printers.printers[key],newState)) {
                        this.printerAction.updateState(key, newState);
                        update = true;
                    }
                }
            }
            this.updatingState = false;
            if(update)
                this.redux.dispatch({type:PrintersAction.SET_STATE_TIME,payload:stateTime});
        });
    };

    sendMoves() {
        if (this.sendMovesCounter == 0) {
            this.connectionHandler.send("sendMoves");
        }
        this.sendMovesCounter++;
    }

    hideMoves() {
        if (this.sendMovesCounter == 1) {
            this.connectionHandler.send("hideMoves");
        }
        if (this.sendMovesCounter > 0)
            this.sendMovesCounter--;
    }
}
