/* (C) 2016 Hot-World GmbH & Co. KG <rlittwin@repetier.com>
 All Rights Reserved

 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY
 DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 It is allowed to use this code to create or use in products connecting to
 Repetier-Server or to use it for own plugins/modules with Repetier-Server.
 This includes free and commercial products.
 Usage for other products is strictly forbidden.
 */

import {
    PrinterState,
    INITIAL_PRINTER_STATE,
    GCodeFileState,
    TemperatureDataState,
    ToolState,
    GroupState,
    LogState
} from "../state/PrinterState";
import {PrinterAction} from "../actions/PrinterAction";
import {RSUtils} from "../RSUtils";

export function PrinterReducer(state: PrinterState = INITIAL_PRINTER_STATE, action: any): PrinterState {
    function orderBy(list: any[], field: string, reverse: boolean) {
        return list.sort(function (a: any[], b: any[]) {
            // ['-online', '-active', 'name']);//r;
            if (reverse) {
                if (a[field] < b[field]) return -1;
                if (a[field] > b[field]) return 1;
            } else {
                if (a[field] < b[field]) return 1;
                if (a[field] > b[field]) return -1;
            }
            return 0;
        });
    }

    function sortModels(state: PrinterState, allModels: GCodeFileState[]): GCodeFileState[] {
        let models: GCodeFileState[] = [];
        let reverse: boolean = (state.sortBy & 1) == 1;
        switch (state.sortBy & 254) {
            case 0:
                models = orderBy(allModels, 'name', reverse);
                break;
            case 2:
                models = orderBy(allModels, 'created', reverse);
                break;
            case 4:
                models = orderBy(allModels, 'printTime', reverse);
                break;
            case 6:
                models = orderBy(allModels, 'length', reverse);
                break;
            case 8:
                models = orderBy(allModels, 'lines', reverse);
                break;
            case 10:
                models = orderBy(allModels, 'filamentTotal', reverse);
                break;
            case 12:
                models = orderBy(allModels, 'layer', reverse);
                break;
        }
        if (state.modelFilter != "") {
            let filtered = [];
            for (let v of models) {
                if (v.name.indexOf(state.modelFilter) >= 0)
                    filtered.push(v);
            }
            models = filtered;
        }
        if (state.activeGroupId != "*") {
            let filtered = [];
            for (let v of models) {
                if (v.group == state.activeGroupId)
                    filtered.push(v);
            }
            models = filtered;
        }
        return models;
    }

    function appendTemperatureEntry(list: TemperatureDataState[], temp: TemperatureDataState): TemperatureDataState[] {
        let start: number = 0;
        while (list.length > start && list[start].time + 300000 < temp.time) {
            start++;
        }
        return list.slice(start).concat(temp);
    }

    let parts: string[] = action.type.split("/");
    if (parts.length > 1) { // aggregated type with pointer

    } else {
        switch (action.type) {
            case PrinterAction.UPDATE_LOG_STATUS: {
                return Object.assign({}, state, {log: Object.assign({}, state.log, action.payload)});
            }
            case PrinterAction.ADD_LOG_LINE: {
                let log: LogState = Object.assign({}, state.log, {lastLogStart: action.payload.id});
                if (!state.log.paused) {
                    let lineText = action.payload.text.replace(/<|>/ig, function (m) {
                        return '&' + (m == '>' ? 'g' : 'l') + 't;';
                    });
                    let newline = action.payload.time + ': ' + lineText;
                    if (log.lastText != lineText) {
                        log.history = log.history.slice(0);
                        log.history.push('<div class="log' + action.payload.type + '">' + newline + '</div>');
                        log.history.splice(0, log.history.length - log.maxLogLines);
                        log.repeat = 1;
                        log.lastText = lineText;
                    } else { // Repeated lines
                        log.repeat++;
                        log.history = log.history.slice(0);
                        log.history[log.history.length - 1] = '<div class="log' + action.payload.type + '">' + newline + ' (' + log.repeat + ')</div>';
                    }
                }
                return Object.assign({}, state, {log: log});
            }
            case PrinterAction.SET_CONFIG: {
                let extra = {config: action.payload.config};
                if (extra.config.extruders.length > state.activeExtruderId) {
                    extra["retractSpeed"] = extra.config.extruders[state.activeExtruderId].retractSpeed;
                    extra["extrudeSpeed"] = extra.config.extruders[state.activeExtruderId].extrudeSpeed;
                }
                return Object.assign({}, state, extra);
            }
            case PrinterAction.RESET_IMAGE_REFERENCE:
                return Object.assign({}, state, {imageUrls: {}});
            case PrinterAction.UPDATE_SCRIPT: {
                let extra = {};
                extra[action.payload.name] = action.payload.script;
                return Object.assign({}, state, {scripts: Object.assign({}, state.scripts, extra)});
            }
            case PrinterAction.SET_GROUPS: {
                let newGroups: GroupState[] = [];
                for (let x in action.payload.groups) {
                    if (action.payload.groups.hasOwnProperty(x))
                        newGroups.push(action.payload.groups[x]);
                }

                return Object.assign({}, state, {
                    groups: action.payload.groups,
                    activeGroupId: action.payload.activeGroup,
                    groupsSorted: orderBy(newGroups, "name", true)
                })
            }
            case PrinterAction.UPDATE_STATE: {
                let newState = Object.assign({}, state, action.payload.state);
                if (state.activeGroupId != newState.activeGroupId) {
                    newState.modelsFiltered = sortModels(newState, newState.allModels);
                }
                return newState;
            }
            case PrinterAction.UPDATE_STATUS:
                return Object.assign({}, state, action.payload.status);
            case PrinterAction.SET_IMG_URL: {
                let a: {[key: string]: string} = {};
                a[action.payload.key] = action.payload.url;
                return Object.assign({}, state, {imageUrls: Object.assign({}, state.imageUrls, a)});
            }
            case PrinterAction.UPDATE_IMG_URLS:
                return Object.assign({}, state, {imageUrls: Object.assign({}, state.imageUrls, action.payload)});
            case PrinterAction.SET_MODELS: {
                return Object.assign({}, state, {
                    allModels: action.payload,
                    modelsFiltered: sortModels(state, action.payload)
                });
            }
            case PrinterAction.SET_PRINTQUEUE:
                return Object.assign({}, state, {queue: action.payload});
            case PrinterAction.ADD_BED_TEMPERATURE: {
                let newBed: ToolState = RSUtils.copyObject(state.heatedBed);
                newBed.tempHistory = appendTemperatureEntry(newBed.tempHistory, action.payload);
                return Object.assign({}, state, {heatedBed: newBed});
            }
            case PrinterAction.ADD_CHAMBER_TEMPERATURE: {
                let newChamber: ToolState = RSUtils.copyObject(state.heatedChamber);
                newChamber.tempHistory = appendTemperatureEntry(newChamber.tempHistory, action.payload);
                return Object.assign({}, state, {heatedChamber: newChamber});
            }
            case PrinterAction.ADD_EXTRUDER_TEMPERATURE: {
                let newExtruders: ToolState[] = RSUtils.copyArray(state.extruders);
                newExtruders[action.payload.extruderId].tempHistory = appendTemperatureEntry(newExtruders[action.payload.extruderId].tempHistory, action.payload.temp);
                return Object.assign({}, state, {extruders: newExtruders});
            }
            case PrinterAction.UPDATE_GCODE: {
                let g: GCodeFileState = action.payload;
                let idx: number = -1;
                let newState: PrinterState = Object.assign({}, state);
                if (g.list == "models") {
                    for (let i: number = 0; i < newState.allModels.length; i++) {
                        if (newState.allModels[i].id == g.id) {
                            idx = i;
                            break;
                        }
                    }
                    if (idx !== -1) {
                        newState.allModels = newState.allModels.slice(0, idx).concat([g], newState.allModels.slice(idx + 1));
                        newState.modelsFiltered = sortModels(newState, newState.allModels);
                    }
                } else {
                    for (let i: number = 0; i < newState.queue.length; i++) {
                        if (newState.queue[i].id == g.id) {
                            idx = i;
                            break;
                        }
                    }
                    if (idx !== -1) {
                        newState.queue = newState.queue.slice(0, idx).concat([g], newState.queue.slice(idx + 1));
                    }
                }
                return newState;
            }
            default:
                return state;
        }
    }
}
